@@ -51,8 +51,9 @@ extern void Done(int, char *) __attribute__((__noreturn__));
 extern void setInitFunctionName(int n, const char *name);
 extern const char *getInitFunctionName(int n);
 extern void CaptureOneWindow(
-  FvwmWindow *fw, Window window, Window keep_on_top_win, Window parent_win);
-extern void CaptureAllWindows(void);
+  FvwmWindow *fw, Window window, Window keep_on_top_win, Window parent_win,
+  Bool is_recapture);
+extern void CaptureAllWindows(Bool is_recapture);
 
 extern XGCValues Globalgcv;
 extern unsigned long Globalgcm;
@@ -64,8 +65,8 @@ extern XContext FvwmContext;
 extern Bool fFvwmInStartup;
 extern Bool DoingCommandLine;
 extern Bool debugging;
+extern Bool debugging_stack_ring;
 extern int GrabPointerState;
-extern Boolean ShapesSupported;
 extern Window JunkRoot, JunkChild;
 extern int JunkX, JunkY;
 extern unsigned int JunkWidth, JunkHeight, JunkBW, JunkDepth, JunkMask;
