@@ -313,13 +313,23 @@ static char *function_vars[] =
   "w.width",
   "w.height",
   "screen",
+  "desk.n",
+  "w.id",
+  "w.name",
+  "w.iconname",
+  "w.class",
+  "w.resource",
+  "version.num",
+  "version.info",
+  "version.line",
+  "desk.pagesx",
+  "desk.pagesy",
   NULL
 };
 
-static int expand_extended_var(
+static signed int expand_extended_var(
   char *var_name, char *output, FvwmWindow *tmp_win)
 {
-  char *s;
   char *rest;
   char dummy[64];
   char *target = (output) ? output : dummy;
@@ -329,6 +339,7 @@ static int expand_extended_var(
   Pixel pixel = 0;
   int val = -12345678;
   Bool is_numeric = False;
+  const char *string = NULL;
 
   /* allow partial matches for *.cs variables */
   switch ((i = GetTokenIndex(var_name, function_vars, -1, &rest)))
@@ -339,14 +350,14 @@ static int expand_extended_var(
   case 3:
     if (!isdigit(*rest) || (*rest == '0' && *(rest + 1) != 0))
       /* not a non-negative integer without leading zeros */
-      return 0;
+      return -1;
     if (sscanf(rest, "%d%n", &cs, &n) < 1)
-      return 0;
+      return -1;
     if (*(rest + n) != 0)
       /* trailing characters */
-      return 0;
+      return -1;
     if (cs < 0)
-      return 0;
+      return -1;
     AllocColorset(cs);
     switch (i)
     {
@@ -421,7 +432,7 @@ static int expand_extended_var(
   case 14:
   case 15:
     if (!tmp_win || IS_ICONIFIED(tmp_win))
-      return 0;
+      return -1;
     else
     {
       is_numeric = True;
@@ -444,7 +455,7 @@ static int expand_extended_var(
 	val = tmp_win->frame_g.height;
 	break;
       default:
-	return 0;
+	return -1;
       }
     }
     break;
@@ -453,29 +464,86 @@ static int expand_extended_var(
     is_numeric = True;
     val = Scr.screen;
     break;
+  case 17:
+    /* desk.n */
+    is_numeric = True;
+    val = Scr.CurrentDesk;
+    break;
+  case 18:
+    /* w.id */
+    if (tmp_win)
+    {
+      sprintf(target, "0x%x", (unsigned int)tmp_win->w);
+      string = target;
+    }
+    break;
+  case 19:
+    /* w.name */
+    if (tmp_win)
+    {
+      string = tmp_win->name;
+    }
+    break;
+  case 20:
+    /* w.iconname */
+    if (tmp_win)
+    {
+      string = tmp_win->icon_name;
+    }
+    break;
+  case 21:
+    /* w.class */
+    if (tmp_win)
+    {
+      string = tmp_win->class.res_class;
+    }
+    break;
+  case 22:
+    /* w.resource */
+    if (tmp_win)
+    {
+      string = tmp_win->class.res_name;
+    }
+    break;
+  case 23:
+    /* version.num */
+    string = VERSION;
+    break;
+  case 24:
+    /* version.info */
+    string = "";
+    break;
+  case 25:
+    /* version.line */
+    string = Fvwm_VersionInfo;
+    break;
+  case 26:
+    /* desk.pagesx */
+    is_numeric = True;
+    val = (int)(Scr.VxMax / Scr.MyDisplayWidth) + 1;
+    break;
+  case 27:
+    /* desk.pagesy */
+    is_numeric = True;
+    val = (int)(Scr.VyMax / Scr.MyDisplayHeight) + 1;
+    break;
   default:
     /* unknown variable - try to find it in the environment */
-    s = getenv(var_name);
-    if (s)
-    {
-      if (output)
-      {
-	strcpy(output, s);
-      }
-      return strlen(s);
-    }
-    else
-    {
-      return 0;
-    }
+    string = getenv(var_name);
   }
   if (is_numeric)
   {
     sprintf(target, "%d", val);
     return strlen(target);
   }
-
-  return 0;
+  else
+  {
+    if (output && string)
+    {
+      strcpy(output, string);
+    }
+    return string ? strlen(string) : -1;
+  }
 }
 
 static char *expand(
@@ -522,7 +590,7 @@ static char *expand(
 	  if (!addto)
 	  {
 	    xlen = expand_extended_var(var, NULL, tmp_win);
-	    if (xlen > 0)
+	    if (xlen >= 0)
 	      l2 += xlen - (k + 2);
 	  }
 	  i += k + 2;
@@ -630,7 +698,7 @@ static char *expand(
 	  k = strlen(var);
 	  xlen = expand_extended_var(var, &out[j], tmp_win);
 	  input[m] = ']';
-	  if (xlen > 0)
+	  if (xlen >= 0)
 	  {
 	    j += xlen;
 	    i += k + 2;
