@@ -117,9 +117,6 @@ typedef struct
 #define WithdrawnState 0
 #endif
 
-#define NO_FOCUS_WIN_EVMASK KeyPressMask|FocusChangeMask
-#define NO_FOCUS_WIN_MENU_EVMASK KeyPressMask|KeyReleaseMask|FocusChangeMask
-
 #if RETSIGTYPE != void
 #define SIGNAL_RETURN return 0
 #else
@@ -133,20 +130,6 @@ typedef struct
 
 #define NULLSTR ((char *) NULL)
 
-typedef struct
-{
-  int x;
-  int y;
-  int width;
-  int height;
-} rectangle;
-
-typedef struct
-{
-  int x;
-  int y;
-} position;
-
 /*
   For 1 style statement, there can be any number of IconBoxes.
   The name list points at the first one in the chain.
@@ -156,7 +139,9 @@ typedef struct icon_boxes_struct
   struct icon_boxes_struct *next;       /* next icon_boxes or zero */
   unsigned int use_count;
   int IconBox[4];                       /* x/y x/y for iconbox */
+  int IconScreen;                       /* Xinerama screen */
   short IconGrid[2];                    /* x incr, y incr */
+  char IconSign[4];                     /* because of -0, need to save */
   unsigned is_orphan : 1;
   unsigned IconFlags : 3;               /* some bits */
   /* IconFill only takes 3 bits.  Defaults are top, left, vert co-ord first */
@@ -199,8 +184,11 @@ typedef struct
     unsigned do_circulate_skip_shaded : 1;
     unsigned do_grab_focus_when_created : 1;
     unsigned do_grab_focus_when_transient_created : 1;
+    unsigned do_iconify_window_groups : 1;
     unsigned do_ignore_gnome_hints : 1;
+    unsigned do_ignore_icon_boxes : 1;
     unsigned do_ignore_restack : 1;
+    unsigned do_use_window_group_hint : 1;
     unsigned do_lower_transient : 1;
     unsigned do_not_show_on_map : 1;
     unsigned do_not_pass_click_focus_click : 1;
@@ -226,6 +214,11 @@ typedef struct
     unsigned has_no_icon_title : 1;
     unsigned has_override_size : 1;
     unsigned has_stippled_title : 1;
+    unsigned icon_override : 2;
+#define NO_ACTIVE_ICON_OVERRIDE 0
+#define ICON_OVERRIDE           1
+#define NO_ICON_OVERRIDE        2
+#define ICON_OVERRIDE_MASK      3
     unsigned is_fixed : 1;
     unsigned is_icon_sticky : 1;
     unsigned is_icon_suppressed : 1;
@@ -245,11 +238,16 @@ typedef struct
   unsigned do_reuse_destroyed : 1;   /* Reuse this struct, don't free it,
 				      * when destroying/recapturing window. */
   unsigned has_border : 1; /* Is this decorated with border*/
+  unsigned has_icon_changed : 1; /* Icon change is pending */
   unsigned has_title : 1; /* Is this decorated with title */
+  unsigned is_decor_changed : 1; /* ChangeDecor was used for window */
   unsigned is_deiconify_pending : 1; /* Sent an XUnmapWindow for deiconifying,
 				      * but didn't receive a UnmapNotify yet.*/
   unsigned is_focused_on_other_desk : 1; /* window had the focus when the desk
 					  * was switched. */
+  /* set if the window was mapped and got focused but the focus change was not
+   * announced to the modules yet. */
+  unsigned is_focus_change_broadcast_pending : 1;
   unsigned is_fully_visible : 1; /* is the window fully visible */
   unsigned is_iconified : 1; /* is it an icon now? */
   unsigned is_iconified_by_parent : 1; /* To prevent iconified transients in a
@@ -272,19 +270,28 @@ typedef struct
   unsigned is_partially_visible : 1; /* is the window partially visible */
   unsigned is_pixmap_ours : 1; /* is the icon pixmap ours to free? */
   unsigned is_placed_wb3 : 1;
+  unsigned is_placed_by_fvwm : 1; /* fvwm2 place the window itself */
   unsigned is_scheduled_for_destroy : 1; /* mark window to be destroyed after
 					  * last complex func has finished. */
   unsigned is_scheduled_for_raise : 1; /* mark window to be raised after
 					* function execution. */
   unsigned is_size_inc_set : 1;
+  unsigned is_style_deleted : 1;
   unsigned is_transient : 1; /* is it a transient window? */
   unsigned is_window_drawn_once : 1;
   unsigned is_viewport_moved : 1; /* To prevent double move in MoveViewport.*/
   unsigned is_window_being_moved_opaque : 1;
+  unsigned is_window_border_drawn : 1; /* 0 at first, set to 1 after border has
+					* been drawn once */
   unsigned is_window_font_loaded : 1;
   unsigned is_window_shaded : 1;
   unsigned using_default_icon_font : 1;
   unsigned using_default_window_font : 1;
+#define ICON_HINT_NEVER    0
+#define ICON_HINT_ONCE     1
+#define ICON_HINT_MULTIPLE 2
+  unsigned was_icon_hint_provided : 2;
+  unsigned was_icon_name_provided : 1;
 } window_flags;
 
 /* Window mask for Circulate and Direction functions */
@@ -292,14 +299,16 @@ typedef struct WindowConditionMask
 {
   struct
   {
+    unsigned do_accept_focus : 1;
     unsigned needs_current_desk : 1;
     unsigned needs_current_page : 1;
+    unsigned needs_current_global_page : 1;
     unsigned needs_name : 1;
     unsigned needs_not_name : 1;
     unsigned use_circulate_hit : 1;
     unsigned use_circulate_hit_icon : 1;
-    unsigned use_circulate_skip : 1;
-    unsigned use_circulate_skip_icon : 1;
+    unsigned use_circulate_hit_shaded : 1;
+    unsigned use_do_accept_focus : 1;
   } my_flags;
   window_flags flags;
   window_flags flag_mask;
@@ -312,12 +321,30 @@ typedef struct
 {
   common_flags_type common;
   unsigned do_decorate_transient : 1;
+  /* old placement flags */
 #define PLACE_DUMB            0x0
 #define PLACE_SMART           0x1
 #define PLACE_CLEVER          0x2
 #define PLACE_CLEVERNESS_MASK 0x3
 #define PLACE_RANDOM          0x4
-#define PLACE_MASK            0x7
+  /* new placements value, try to get a minimal backward compatibility with
+   *  the old flags: Dumb+Active=Manual, Dumb+Random=Cascade, Smart+Random=
+   *  TileCascade, Smart+Active=TileManual, Random+Smart+Clever=MINOVERLAP
+   *  which is the original Clever placement code, Active+Smart+Clever=
+   *  MINOVERLAPPERCENT which is the "new" Clever placement code and was
+   *  the original Clever placement code. Now the original placement code
+   *  said: Active/Random+Dumb+Clever=Active/Random+Dumb (with Dumb Clever
+   *  is ignored); These represent the not use value: 0x2=Active+Dumb+Clever,
+   *  0x6=Random+Dumb+Clever */
+#define PLACE_MANUAL            0x0
+#define PLACE_TILEMANUAL        0x1
+#define PLACE_MANUAL_B          0x2
+#define PLACE_MINOVERLAPPERCENT 0x3
+#define PLACE_CASCADE           0x4
+#define PLACE_TILECASCADE       0x5
+#define PLACE_CASCADE_B         0x6
+#define PLACE_MINOVERLAP        0x7
+#define PLACE_MASK              0x7
   unsigned placement_mode : 3;
   unsigned do_save_under : 1;
   unsigned do_start_lowered : 1;
@@ -344,7 +371,11 @@ typedef struct
   unsigned has_condition_mask : 1;
 #endif
   unsigned is_button_disabled : NUMBER_OF_BUTTONS;
-  unsigned use_backing_store : 1;
+#define BACKINGSTORE_DEFAULT 0
+#define BACKINGSTORE_ON      1
+#define BACKINGSTORE_OFF     2
+#define BACKINGSTORE_MASK  0x3
+  unsigned use_backing_store : 2;
   unsigned use_parent_relative : 1;
   unsigned use_colorset : 1;
   unsigned use_colorset_hi : 1;
@@ -352,16 +383,15 @@ typedef struct
   unsigned use_border_colorset_hi : 1;
   unsigned use_layer : 1;
   unsigned use_no_pposition : 1;
+  unsigned use_no_usposition : 1;
+  unsigned use_no_transient_pposition : 1;
+  unsigned use_no_transient_usposition : 1;
   unsigned use_start_on_desk : 1;
   unsigned use_start_on_page_for_transient : 1;
-  unsigned active_placement_honors_starts_on_page : 1;
+  unsigned use_start_on_screen : 1;
+  unsigned manual_placement_honors_starts_on_page : 1;
   unsigned capture_honors_starts_on_page : 1;
   unsigned recapture_honors_starts_on_page : 1;
-  unsigned icon_override : 2;
-#define NO_ACTIVE_ICON_OVERRIDE 0
-#define ICON_OVERRIDE           1
-#define NO_ICON_OVERRIDE        2
-#define ICON_OVERRIDE_MASK      3
 } style_flags;
 
 /* only style.c and add_window.c are allowed to access this struct!! */
@@ -396,6 +426,7 @@ typedef struct window_style
   int start_desk;
   int start_page_x;
   int start_page_y;
+  int start_screen;
   int max_window_width;
   int max_window_height;
   int shade_anim_steps;
@@ -445,15 +476,16 @@ typedef struct FvwmWindow
 #endif
   Window icon_w;              /* the icon window */
   Window icon_pixmap_w;       /* the icon window */
-#ifdef SHAPE
   int wShaped;               /* is this a shaped window */
-#endif
+  Pixmap title_background_pixmap;
+  Pixmap button_background_pixmap[NUMBER_OF_BUTTONS];
 
   short boundary_width;
   short corner_width;
   short visual_corner_width;
 
   rectangle title_g;
+  short title_top_height;     /* title height/0 for top/bottom titles */
   rectangle icon_g;           /* geometry of the icon window */
   int icon_xl_loc;            /* icon label window x coordinate */
   int icon_t_width;           /* width of the icon title window */
@@ -482,6 +514,8 @@ typedef struct FvwmWindow
   rectangle frame_g;
   rectangle normal_g;         /* absolute geometry when not maximized */
   rectangle max_g;            /* maximized window geometry */
+  size_rect max_g_defect;     /* defect between maximized geometry before and
+			       * after constraining size. */
   position max_offset;        /* original delta between normalized and
 			       * maximized window, used to keep unmaximized
 			       * window at same screen position */
@@ -505,11 +539,26 @@ typedef struct FvwmWindow
   int shade_anim_steps;
   unsigned char grabbed_buttons;
 
-  void *pscratch;             /* multi purpose scratch pointer */
+#define FM_NO_INPUT        0
+#define FM_PASSIVE         1
+#define FM_LOCALLY_ACTIVE  2
+#define FM_GLOBALLY_ACTIVE 3
+  unsigned char focus_model;
+
+  unsigned char initial_backing_store;
+
+  /* multi purpose scratch structure */
+  struct
+  {
+    void *p;
+    int i;
+  } scratch;
 } FvwmWindow;
 
 /* include this down here because FvwmWindows must be defined when including
  * this header file. */
 #include "fvwmdebug.h"
 
+void SetMWM_INFO(Window window);
+
 #endif /* _FVWM_ */
