@@ -442,10 +442,16 @@ void CMD_DesktopSize(F_CMD_ARGS)
  *
  * Check to see if the pointer is on the edge of the screen, and scroll/page
  * if needed
+ *
+ * Returns
+ *  0: no paging
+ *  1: paging occured
+ * -1: no need to call the function again before a new event arrives
  ***************************************************************************/
-Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
-		  int *delta_x, int *delta_y, Bool Grab, Bool fLoop,
-		  Bool do_continue_previous)
+int HandlePaging(
+	int HorWarpSize, int VertWarpSize, int *xl, int *yt,
+	int *delta_x, int *delta_y, Bool Grab, Bool fLoop,
+	Bool do_continue_previous)
 {
   static unsigned int add_time = 0;
   int x,y;
@@ -459,30 +465,38 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 
   *delta_x = 0;
   *delta_y = 0;
-
-  /* Only continue to count the time, but do not start counting from
-   * scratch. */
   if (!is_timestamp_valid && do_continue_previous)
   {
-    /* Free some CPU */
-    usleep(10000);
-    return False;
+    /* don't call me again until something has happened */
+    return -1;
+  }
+  if (!is_timestamp_valid && Event.type == MotionNotify)
+  {
+    x = Event.xmotion.x_root;
+    y = Event.xmotion.y_root;
+    /* need to move the viewport */
+    if ((Scr.VxMax == 0 ||
+	 (x >= edge_thickness &&
+	  x < Scr.MyDisplayWidth  - edge_thickness)) &&
+	(Scr.VyMax == 0 ||
+	 (y >= edge_thickness &&
+	  y < Scr.MyDisplayHeight - edge_thickness)))
+    {
+      return -1;
+    }
   }
-
   if (Scr.ScrollResistance >= 10000 || (HorWarpSize == 0 && VertWarpSize==0))
   {
     is_timestamp_valid = False;
     add_time = 0;
-    return False;
+    return 0;
   }
-
   if (Scr.VxMax == 0 && Scr.VyMax == 0)
   {
     is_timestamp_valid = False;
     add_time = 0;
-    return False;
+    return 0;
   }
-
   if (!is_timestamp_valid)
   {
     is_timestamp_valid = True;
@@ -513,15 +527,14 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
       StashEventTime(&Event);
       is_timestamp_valid = False;
       add_time = 0;
-      return False;
+      return 0;
     }
-    else if (XCheckMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask |
-			     KeyPressMask | KeyReleaseMask, &e))
+    else if (XCheckMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask, &e))
     {
       XPutBackEvent(dpy, &e);
       is_timestamp_valid = False;
       add_time = 0;
-      return False;
+      return 0;
     }
     /* get pointer location */
     GetLocationFromEventOrQuery(dpy, Scr.Root, &Event, &x, &y);
@@ -536,7 +549,7 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
       discard_window_events(Scr.PanFrameBottom.win, LeaveWindowMask);
       discard_window_events(Scr.PanFrameLeft.win, LeaveWindowMask);
       discard_window_events(Scr.PanFrameRight.win, LeaveWindowMask);
-      return False;
+      return 0;
     }
     if (!fLoop && is_last_position_valid &&
 	(x - last_x > MAX_PAGING_MOVE_DISTANCE ||
@@ -552,7 +565,7 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
       add_time = 0;
       last_x = x;
       last_y = y;
-      return False;
+      return 0;
     }
     last_x = x;
     last_y = y;
@@ -564,7 +577,7 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 
   if (lastTimestamp - my_timestamp + add_time < Scr.ScrollResistance)
   {
-    return False;
+    return 0;
   }
 
   /* Move the viewport */
@@ -659,7 +672,7 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
   add_time = 0;
   if (*delta_x == 0 && *delta_y == 0)
   {
-    return False;
+    return 0;
   }
 
   if(Grab)
@@ -673,7 +686,7 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
   if(Grab)
     MyXUngrabServer(dpy);
 
-  return True;
+  return 1;
 }
 
 
@@ -983,7 +996,7 @@ void MoveViewport(int newx, int newy, Bool grab)
      * domivogt (29-Nov-1999): It's faster to first map windows top to bottom
      * and then unmap windows bottom up.
      */
-    t = Scr.FvwmRoot.stack_next;
+    t = get_next_window_in_stack_ring(&Scr.FvwmRoot);
     while (t != &Scr.FvwmRoot)
     {
       /*
@@ -1039,9 +1052,9 @@ void MoveViewport(int newx, int newy, Bool grab)
 	}
       }
       /*  Bump to next win...	 */
-      t = t->stack_next;
+      t = get_next_window_in_stack_ring(t);
     }
-    t1 = Scr.FvwmRoot.stack_prev;
+    t1 = get_prev_window_in_stack_ring(&Scr.FvwmRoot);
     while (t1 != &Scr.FvwmRoot)
     {
       /*
@@ -1090,7 +1103,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 	}
       }
       /*  Bump to next win...	 */
-      t1 = t1->stack_prev;
+      t1 = get_prev_window_in_stack_ring(t1);
     }
     for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
     {
@@ -1141,7 +1154,8 @@ static void UnmapDesk(int desk, Bool grab)
   {
     MyXGrabServer(dpy);
   }
-  for (t = Scr.FvwmRoot.stack_prev; t != &Scr.FvwmRoot; t = t->stack_prev)
+  for (t = get_prev_window_in_stack_ring(&Scr.FvwmRoot); t != &Scr.FvwmRoot;
+       t = get_prev_window_in_stack_ring(t))
   {
     /* Only change mapping for non-sticky windows */
     if(!(IS_ICONIFIED(t) && IS_ICON_STICKY(t)) &&
@@ -1196,7 +1210,8 @@ static void MapDesk(int desk, Bool grab)
   {
     MyXGrabServer(dpy);
   }
-  for (t = Scr.FvwmRoot.stack_next; t != &Scr.FvwmRoot; t = t->stack_next)
+  for (t = get_next_window_in_stack_ring(&Scr.FvwmRoot); t != &Scr.FvwmRoot;
+       t = get_next_window_in_stack_ring(t))
   {
     /* Only change mapping for non-sticky windows */
     if(!(IS_ICONIFIED(t) && IS_ICON_STICKY(t)) &&
@@ -1330,7 +1345,12 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
     val[1] = prev_desk_and_page_page_x;
     val[2] = prev_desk_and_page_page_y;
   }
-  else if (GetIntegerArguments(action, NULL, val, 3) != 3)
+  else if (GetIntegerArguments(action, NULL, val, 3) == 3)
+  {
+    val[1] *= Scr.MyDisplayWidth;
+    val[2] *= Scr.MyDisplayHeight;
+  }
+  else
   {
     return;
   }
@@ -1342,7 +1362,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
   }
   prev_desk_and_page_page_x = Scr.Vx;
   prev_desk_and_page_page_y = Scr.Vy;
-  MoveViewport(val[1] * Scr.MyDisplayWidth, val[2] * Scr.MyDisplayHeight, True);
+  MoveViewport(val[1], val[2], True);
   if (is_new_desk)
   {
     prev_desk = Scr.CurrentDesk;
