@@ -393,6 +393,7 @@ int GetEqualSizeChildren(
 #define C_UNMANAGED  0x10000
 #define C_RALL       (C_R1|C_R2|C_R3|C_R4|C_R5)
 #define C_LALL       (C_L1|C_L2|C_L3|C_L4|C_L5)
+#define C_DECOR      (C_LALL|C_RALL|C_TITLE|C_FRAME|C_SIDEBAR)
 #define C_ALL   (C_WINDOW|C_TITLE|C_ICON|C_ROOT|C_FRAME|C_SIDEBAR|\
                  C_LALL|C_RALL)
 
@@ -477,7 +478,7 @@ KeySym FvwmStringToKeysym(Display *dpy, char *key);
 
 void fvwmlib_keyboard_shortcuts(
     Display *dpy, int screen, XEvent *Event, int x_move_size, int y_move_size,
-    int ReturnEvent);
+    int *x_defect, int *y_defect, int ReturnEvent);
 
 void fvwmlib_get_target_window(
     Display *dpy, int screen, char *MyName, Window *app_win,
