@@ -505,14 +505,14 @@ static void ct_Font(char *cp)
 {
   if (font_names[f_text])
     free(font_names[f_text]);
-  font_names[f_text] = safestrdup(cp);
+  CopyStringWithQuotes(&font_names[f_text], cp);
   myfprintf((stderr, "Font: %s\n", font_names[f_text]));
 }
 static void ct_ButtonFont(char *cp)
 {
   if (font_names[f_button])
     free(font_names[f_button]);
-  font_names[f_button] = safestrdup(cp);
+  CopyStringWithQuotes(&font_names[f_button], cp);
   myfprintf((stderr, "ButtonFont: %s\n", font_names[f_button]));
 }
 static void ct_ButtonInPointer(char *cp)
@@ -539,7 +539,7 @@ static void ct_InputFont(char *cp)
 {
   if (font_names[f_input])
     free(font_names[f_input]);
-  font_names[f_input] = safestrdup(cp);
+  CopyStringWithQuotes(&font_names[f_input], cp);
   myfprintf((stderr, "InputFont: %s\n", font_names[f_input]));
 }
 static void ct_InputPointer(char *cp)
