@@ -210,7 +210,7 @@ Bool GoodiesParseConfig(char *tline)
     UpdateString(&clockfmt, rest);
     break;
   case 4: /* StatusFont */
-    CopyString(&statusfont_string, rest);
+    CopyStringWithQuotes(&statusfont_string, rest);
     break;
   case 5: /* TipsFore */
     CopyString(&TipsFore, rest);
@@ -248,8 +248,6 @@ void InitGoodies(void)
   char tmp[1024];
 #ifdef I18N_MB
   char **ml;
-  int mc;
-  char *ds;
   XFontStruct **fs_list;
 #endif
 
@@ -261,15 +259,10 @@ void InitGoodies(void)
   }
 
 #ifdef I18N_MB
-  if ((StatusFontset=XCreateFontSet(dpy,statusfont_string,&ml,&mc,&ds))==NULL) {
-#ifdef STRICTLY_FIXED
-    if ((StatusFontset=XCreateFontSet(dpy,"-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",&ml,&mc,&ds))==NULL) {
-#else
-    if ((StatusFontset=XCreateFontSet(dpy,"fixed,-*--14-*",&ml,&mc,&ds))==NULL) {
-#endif
-      fprintf(stderr, "%s: Couldn't load fixed fontset...exiting !\n",Module);
-      exit(1);
-    }
+  if ((StatusFontset=GetFontSetOrFixed(dpy,statusfont_string)) == NULL)
+  {
+	  fprintf(stderr, "%s: Couldn't load status font exiting !\n",Module);
+	  exit(1);
   }
   XFontsOfFontSet(StatusFontset,&fs_list,&ml);
   StatusFont = fs_list[0];
