@@ -61,6 +61,7 @@
 #include <string.h>
 #include <sys/wait.h>
 #include <sys/time.h>
+#include <time.h>
 
 #if HAVE_SYS_SELECT_H
 #include <sys/select.h>
@@ -70,6 +71,8 @@
 #include <ctype.h>
 #include <stdlib.h>
 #include "libs/fvwmlib.h"
+#include "libs/FScreen.h"
+#include "libs/FShape.h"
 #include "libs/Module.h"
 #include "libs/Picture.h"
 #include "libs/Colorset.h"
@@ -267,8 +270,10 @@ int main(int argc, char **argv)
     exit (1);
   }
   InitPictureCMap(dpy);
+  FScreenInit(dpy);
   /* Initialise default colorset */
   AllocColorset(0);
+  FShapeInit(dpy);
 
   x_fd = XConnectionNumber(dpy);
 
@@ -510,6 +515,7 @@ void Loop(void)
 	if (Event.xbutton.button != Button1) {
 	  if (Event.xbutton.button == Button2) {
 	    static int LastX, LastY;
+            static int scr_x, scr_y, scr_w, scr_h;
 
 	    if (LastMapped != -1) {
 	      CloseFolder(LastMapped);
@@ -529,6 +535,7 @@ void Loop(void)
 	      Window junk;
 	      unsigned int junk2,junk3,junk4,junk5;
 	      int CornerX, CornerY;
+	      fscreen_scr_arg fscr;
 
 #ifdef ENABLE_SOUND
 	      PlaySound(WHEV_CLOSE_MAIN);
@@ -541,17 +548,24 @@ void Loop(void)
 	      }
 	      XTranslateCoordinates(
 		dpy,main_win,Root,LastX,LastY,&LastX,&LastY,&junk);
+              /*!!! Note: maybe it will be better to use (LastX+W/2,LastY+H/2)
+                in ScrRect query? That could help in pathological cases when
+                Wharf is not completely on a single screen. */
+    	      fscr.xypos.x = LastX;
+	      fscr.xypos.y = LastY;
+              FScreenGetScrRect(
+		&fscr, FSCREEN_XYPOS, &scr_x, &scr_y, &scr_w, &scr_h);
 	      if (num_rows<num_columns) { /* horizontal */
-		if (LastY > display_height/2) {
-		  CornerY = display_height-BUTTONHEIGHT;
+		if (LastY > scr_y + scr_h / 2) {
+		  CornerY = scr_y+scr_h-BUTTONHEIGHT;
 		} else {
-		  CornerY = 0;
+		  CornerY = scr_y;
 		}
 		if (Event.xbutton.x>num_columns*BUTTONWIDTH/2) {
-		  CornerX = display_width - BUTTONWIDTH;
+		  CornerX = scr_x + scr_w - BUTTONWIDTH;
 		  AnimationDir = DIR_TOLEFT;
 		} else {
-		  CornerX = 0;
+		  CornerX = scr_x;
 		  AnimationDir = DIR_TORIGHT;
 		}
 		if (AnimationStyle>0 && AnimateMain) {
@@ -563,16 +577,16 @@ void Loop(void)
 				    BUTTONWIDTH,BUTTONHEIGHT);
 		}
 	      } else {	/* vertical */
-		if (LastX > display_width/2) {
-		  CornerX = display_width - BUTTONWIDTH;
+		if (LastX > scr_x + scr_w/2) {
+		  CornerX = scr_x + scr_w - BUTTONWIDTH;
 		} else {
-		  CornerX = 0;
+		  CornerX = scr_x;
 		}
 		if (Event.xbutton.y>num_rows*BUTTONHEIGHT/2) {
-		  CornerY = display_height-BUTTONHEIGHT;
+		  CornerY = scr_y + scr_h - BUTTONHEIGHT;
 		  AnimationDir = DIR_TOUP;
 		} else {
-		  CornerY = 0;
+		  CornerY = scr_y;
 		  AnimationDir = DIR_TODOWN;
 		}
 		if (AnimationStyle>0 && AnimateMain) {
@@ -1042,6 +1056,13 @@ void MapFolder(int folder, int *LastMapped, int base_x, int base_y, int row,
   else
   {
     int folderx, foldery, folderw, folderh;
+    int scr_x, scr_y, scr_w, scr_h;
+    fscreen_scr_arg fscr;
+
+    fscr.xypos.x = base_x;
+    fscr.xypos.y = base_y;
+    FScreenGetScrRect(&fscr, FSCREEN_XYPOS, &scr_x, &scr_y, &scr_w, &scr_h);
+
     if (*LastMapped != -1)
     {
       CloseFolder(*LastMapped);
@@ -1051,7 +1072,7 @@ void MapFolder(int folder, int *LastMapped, int base_x, int base_y, int row,
     Folders[folder].mapped = ISMAPPED;
     if(num_columns < num_rows)
     {
-      if((base_x % display_width) > display_width / 2 ) {
+      if((base_x % display_width) > scr_x + scr_w / 2 ) {
 	folderx = base_x+(col-Folders[folder].count)*BUTTONWIDTH-2;
 	dir = DIR_TOLEFT;
       }
@@ -1074,7 +1095,7 @@ void MapFolder(int folder, int *LastMapped, int base_x, int base_y, int row,
       */
       if (ROWS)
       {
-	if ((base_y % display_height) > display_height / 2) {
+	if ((base_y % display_height) > scr_y + scr_h / 2) {
 	  foldery = base_y-(Folders[folder].count)*BUTTONHEIGHT-2;
 	  dir = DIR_TOUP;
 	}
@@ -1088,7 +1109,7 @@ void MapFolder(int folder, int *LastMapped, int base_x, int base_y, int row,
       }
       else
       {
-	if((base_x % display_width) > display_width / 2 ) {
+	if((base_x % display_width) > scr_x + scr_w / 2 ) {
 	  folderx = base_x-(Folders[folder].count)*BUTTONWIDTH-2;
 	  dir = DIR_TOLEFT;
 	}
@@ -1103,7 +1124,7 @@ void MapFolder(int folder, int *LastMapped, int base_x, int base_y, int row,
     }
     else
     {
-      if ((base_y % display_height) < display_height / 2) {
+      if ((base_y % display_height) < scr_y + scr_h / 2) {
 	foldery  =base_y+(row+1)*BUTTONHEIGHT;
 	dir = DIR_TODOWN;
       }
@@ -1430,27 +1451,31 @@ void CreateWindow(void)
   mysizehints.height_inc = BUTTONHEIGHT;
   mysizehints.base_height = 0;
   mysizehints.base_width = 0;
-  if(x > -100000)
+  if (x > -100000)
   {
-    if (x <= -1)
+    if (flags & XNegative)
     {
-      mysizehints.x = DisplayWidth(dpy,screen) + x - mysizehints.width-1;
+      mysizehints.x = DisplayWidth(dpy,screen) + x - mysizehints.width;
       gravity = NorthEastGravity;
     }
-    else if ((x == 0) && (flags & 16))
-      mysizehints.x = DisplayWidth(dpy,screen) - mysizehints.width-2;
     else
-      mysizehints.x = x;
-    if ( y<0)
     {
-      mysizehints.y = DisplayHeight(dpy,screen) + y - mysizehints.height-2;
+      mysizehints.x = x;
+    }
+    if (flags & YNegative)
+    {
+      mysizehints.y = DisplayHeight(dpy,screen) + y - mysizehints.height;
       gravity = SouthWestGravity;
     }
     else
+    {
       mysizehints.y = y;
+    }
 
-    if((x < 0) && (y < 0))
+    if ((flags & XNegative) && (flags & YNegative))
+    {
       gravity = SouthEastGravity;
+    }
     mysizehints.flags |= USPosition;
   }
 
@@ -1571,7 +1596,7 @@ void ParseOptions(char *filename)
 	if (isspace(tmp[strlen(tmp)-1]))
 	  tmp[strlen(tmp)-1] = 0;
 
-	flags = XParseGeometry(tmp,&g_x,&g_y,&width,&height);
+	flags = FScreenParseGeometry(tmp,&g_x,&g_y,&width,&height);
 	if (flags & WidthValue)
 	  w = width;
 	if (flags & HeightValue)
@@ -1762,6 +1787,12 @@ void ParseOptions(char *filename)
       /* store colorset sent by fvwm */
       LoadColorset(&(tline[8]));
     }
+    else if((strncasecmp(tline, XINERAMA_CONFIG_STRING,
+			 sizeof(XINERAMA_CONFIG_STRING) - 1)==0))
+    {
+      FScreenConfigureModule(
+	tline + sizeof(XINERAMA_CONFIG_STRING) - 1);
+    }
     GetConfigLine(fd, &tline);
     orig_tline = tline;
   } /* while */
@@ -2236,7 +2267,7 @@ static void change_colorset(int colorset)
   return;
 }
 
-static void handle_colorset_packet(unsigned long *body)
+static void handle_config_info_packet(unsigned long *body)
 {
   char *tline, *token;
   int colorset;
@@ -2248,6 +2279,10 @@ static void handle_colorset_packet(unsigned long *body)
     colorset = LoadColorset(tline);
     change_colorset(colorset);
   }
+  else if (StrEquals(token, XINERAMA_CONFIG_STRING))
+  {
+    FScreenConfigureModule(tline);
+  }
 
   return;
 }
@@ -2281,7 +2316,7 @@ void process_message(unsigned long type,unsigned long *body)
     CheckForHangon(body);
     break;
   case M_CONFIG_INFO:
-    handle_colorset_packet((unsigned long*)body);
+    handle_config_info_packet((unsigned long*)body);
     break;
   default:
     break;
